from typing import Tuple
from typing import Union

from compas.geometry import Point
from compas.geometry import Vector
from compas.geometry import Frame
from compas.geometry import Curve
from compas.geometry import Box
from compas.geometry import Transformation

from OCC.Core.Geom import Geom_Curve
from OCC.Core.TopoDS import TopoDS_Shape
from OCC.Core.TopoDS import TopoDS_Edge


class OCCCurve(Curve):

    def __init__(self, name=None) -> None: ...

    def __eq__(self, other: 'OCCCurve') -> bool: ...

    @property
    def occ_curve(self) -> Geom_Curve: ...

    @occ_curve.setter
    def occ_curve(self, curve: Geom_Curve): ...

    @property
    def occ_shape(self) -> TopoDS_Shape: ...

    @property
    def occ_edge(self) -> TopoDS_Edge: ...

    @property
    def dimension(self) -> int: ...

    @property
    def domain(self) -> Tuple[float, float]: ...

    @property
    def start(self) -> Point: ...

    @property
    def end(self) -> Point: ...

    @property
    def is_closed(self) -> bool: ...

    @property
    def is_periodic(self) -> bool: ...

    @classmethod
    def from_occ(cls, occ_curve: Geom_Curve) -> 'OCCCurve': ...

    def to_step(self, filepath: str, schema: str = "AP203") -> None: ...

    def transform(self, T: Transformation) -> None: ...

    def reverse(self) -> None: ...

    def point_at(self, t: float) -> Point: ...

    def tangent_at(self, t: float) -> Vector: ...

    def curvature_at(self, t: float) -> Vector: ...

    def frame_at(self, t: float) -> Frame: ...

    def aabb(self, precision: float = 0.0) -> Box: ...

    def length(self, precision: float = 1e-3) -> float: ...

    def closest_point(self, point: Point, return_parameter: bool = False) -> Union[Point, Tuple[Point, float]]: ...

    def closest_parameters_curve(self, curve: 'OCCCurve', return_distance: bool = False) -> Union[Tuple[float, float], Tuple[Tuple[float, float], float]]: ...

    def closest_points_curve(self, curve: 'OCCCurve', return_distance: bool = False) -> Union[Tuple[Point, Point], Tuple[Tuple[Point, Point], float]]: ...
